/*****************************************************
Workspace: CVAVR
Chip type: ATmega8
Project  : PPGDT soldering station
Date     : Nov, 2018
*****************************************************/
#include <mega8.h>
#include <stdlib.h>
#include <math.h>
#include <delay.h>
#include <stdbool.h>
#include <stdlib.h>
#include "defines.c"
#include "periph_init.c"
#include "keywork.c"
#include "display_work.c"
#include "ee_work.c"
#include "pwm_work.c"
#include "adc_work.c"
#include "pow.c"

u16 tim0, //msc 
    tim1, //adc reading timer
    tim2, //timeout_m
    tim3; //encoder dt 
u8  tim_s1;  //200ms timer    
u16 tim_tcp; //tcp err timer 
    
bit iwdg_t2;

interrupt [TIM1_OVF] void t1_ovf_isr()
{
 TIMSK &= ~(1<<TOIE1);
    
 if(tmpB != (u8)(cpu_speed/(pwm_freq*2))-tmpA)  
 {
  PORTC.5 = 0; 
 }   
 else 
 {
  OCR1A = tmpA; 
  OCR1B = tmpB;
 } 
 
 t1_trg=1;
}
interrupt [TIM2_COMP] void t2_ISR()
{
 static bit isr_trg;
 
 if(isr_trg)
 {
  isr_trg=0;
  scan_keys();   
 }
 else
 {
  isr_trg=1;
 }

 if(tim0)tim0--;
 if(tim1)tim1--; 
 if(tim2)tim2--; 
 if(tim3)tim3--; 
      
 if(tim_s1)tim_s1--;
 if(tim_tcp)tim_tcp--; 
 if(kb_timer)kb_timer--; 
 
 iwdg_t2=1;
}

void devcon();

void main(void)
{
 gpio_init();
 t1_init(); 
 t2_init();
 adc_init();
 ee_init();
 detect_7seg_polarity(); 

 #pragma optsize-
 WDTCR=0x18;  //16ms
 WDTCR=0x08;
 #ifdef _OPTIMIZE_SIZE_
 #pragma optsize+
 #endif

 #asm("sei") 
 
 infinitely()
 { 
  devcon();
  display_data();  
      
  if(iwdg_t2){iwdg_t2=0; #asm("wdr")}
 }
}

void devcon()
{
 static enum {start,norm,sdn,tc_err,sc_prot,timeout_sdn,setup}dev_st=start;
 static bit itr=0;
 static u16 iron_t_def; 
 static u16 iron_t_tmp; 
 u8 i; 
 u16 tmp;
 
 static bit skb_trg; 
 if(btn[0]==rls && btn[1]==rls)skb_trg=0;

switch(dev_st)
{
 case start:
 {
  static bit sts;
  if(!itr)
  {
   itr=1;    
   dbuf[2]='t';dbuf[1]='c';dbuf[0]=0;dpp=1;
   tim0=1000;
   set_pwm(PWM_DEFVAL); 
   sts=0;
   pm = ee; 
  }

  if(!tim0)
  {
   for(i=0;i<noc;i++)
   if(btn[i]==prs || btn[i]==hold)
   {
    dev_st=setup;
    skb_trg=1;
    goto endofstart;
   }  
   
   if(!sts)
   {
    iron_t_def = ee_trd ()*10; 
    if(iron_t_def < 150) iron_t_def = 150;
    convert_data(iron_t_def);
    dpp=0; 
    tim0=1000;    
    sts++;
   } 
   else dev_st=norm;
  }  
  
endofstart:  
  if(dev_st!=start){itr=0;}      
 }
 break;

 case norm:                 
 {
  static u8  timeout_m;
  static s16 tc_data[CO];
  static u8  cmp2ee; 
  static bit ee_wt; 
  static u16 adc_tmp; 

  if(!itr)
  {
   itr=1;    
   
   timeout_m = 0xff;
   cmp2ee = iron_t_def/10;

   tim3 = 2000;            //          
   tim_tcp = 10000; 
  }  
  
  //   
  if(!tim1)  
  { 
   static bit trg;
   
   if(!trg)
   { 
    trg = 1;
        
//    PWM_OFF;
//    delay_us(60); 
   }
     
   //     
   if(read_adc(&adc_tmp)) 
   { 
    tim1 = 500; 
    trg = 0;
//    PWM_ON; 
   
    //        
    static u8 tc_cntdn=5;
  
    adc_tmp = adc_tmp*pm.cpadc;          //  *100
                                                              
    for(i=CO-1;i;)tc_data[i]=tc_data[--i]; //      
           
    //          
    if(adc_tmp > pm.offset) 
//    tc_data[0] = (AMBIENT*100 +adc_tmp)/100; //         
    tc_data[0] = (AMBIENT*100 +adc_tmp -pm.offset)/100;
    else 
    tc_data[0] = AMBIENT;  
    
    //    
    //     
    if(tc_data[0] < MAXTC/100-1 && tc_data[0] > AMBIENT)  
    for(i=0;i<CO-1;i++) if(tc_data[i]!=tc_data[i+1])tc_cntdn = 5*CO;      

    if(!tc_cntdn--)dev_st = tc_err;
    //        
       
    //   
    set_pwm(t_stab_proc(iron_t_tmp, tc_data));   
   } 

   //        
  }  
  
  //        
  if(1)
  {   
   //  
   kb_input(&iron_t_def,100,450,50,5,0xff); // 

   if(iron_t_def!=iron_t_tmp) //   t
   {          
    if(iron_t_def < 150) 
    {   
     skb_trg = 1;
     dev_st = sdn; 
    }
    else
    {
     iron_t_tmp = iron_t_def;
     tim3 = 2000;             //   
     ee_wt=1;                 //    eeprom
     timeout_m = 0xff;        //         
    }
   }    
   //   
  
   //    
   if(!tim3)     //       
   {  
    static s16 dtmp; 
    static u32 tmp;
    
    tmp = 0;
    for(i=0;i<CO;i++) tmp +=tc_data[i]; 
    tmp /= CO;
    
    //  
    //           
    if(dtmp != tmp || !dpp) 
    {
     dtmp = tmp;
     dpp=5; 
     convert_data(tmp);
    }
     
    //   eeprom   
    if(ee_wt)                           
    { 
     ee_wt = 0;     
     tmp = iron_t_tmp/10;
     if(tmp != cmp2ee) 
     {
      ee_t_wt(tmp);  
      cmp2ee = tmp; 
     }
    }
   }
   else 
   {
    dpp=0;
    convert_data(iron_t_tmp);  //     
   }
   //        
  
   //  
   if(!tim2) //1min 
   {  
    tim2 = 60000; 
   
    if(timeout_m)
    {   
     if(tc_data[0] > 400) 
     {
      if(timeout_m > 5) timeout_m-=5;
      else timeout_m = 0;
     }
     else timeout_m--;
    }
    else dev_st = timeout_sdn;  
   } 
   //   
  }
  

  if(dev_st!=norm){itr=0;}  
 }
 break;

 case sdn:
 {
  if(!itr)
  {
   itr=1;
   set_pwm(PWM_DEFVAL); 
   iron_t_tmp=iron_t_def;   
   dbuf[2]=dbuf[1]=dbuf[0]='_';dpp=5;   
  } 
       
  if(!skb_trg)
  {
   kb_input(&iron_t_tmp,100,450,50,5,0xff); 
   if(iron_t_tmp != iron_t_def) dev_st=start; 
  }

  if(dev_st!=sdn){itr=0;}    
 }
 break;  
 
 case tc_err:
 {
  static u16 pp;
  if(!itr)
  {
   itr=1;  
   dbuf[2]='t';dbuf[1]='c';dbuf[0]='E';dpp=1;
   pp = 10;
   set_pwm(PWM_DEFVAL);  
  }   
  
  tmp=pp;
  kb_input(&pp,10,100,10,1,0xff); 
  
  if(tmp!=pp)
  {
   convert_data(pp); 
   dpp=3; dbuf[2]='P';
   set_pwm(pp);
  }   

  if(dev_st!=tc_err){itr=0;}   
 } 
 break;

 case sc_prot:
 {
  if(!itr)
  {
   itr=1;
   set_pwm(PWM_DEFVAL);  
   dbuf[2]='S';dbuf[1]='c';dbuf[0]='P';dpp=1;    
  }  

  if(dev_st!=sc_prot){itr=0;}   
 } 
 break;
 
 case timeout_sdn:
 {
  if(!itr)
  {
   itr=1;
   set_pwm(PWM_DEFVAL);
   iron_t_tmp=iron_t_def; 
   dbuf[2]='t';dbuf[1]='o';dbuf[0]=' ';dpp=0;    
  }  
  
  kb_input(&iron_t_tmp,100,450,50,5,0xff); 
  if(iron_t_tmp != iron_t_def) dev_st=start;

  if(dev_st!=timeout_sdn){itr=0;}   
 } 
 break;
 
 case setup:
 {
  static enum {set_amp, set_offset, set_irres}setting;
  static bit stt_trg; 

  if(!itr)
  {
   itr=1;
   dbuf[2]='S';dbuf[1]='E';dbuf[0]='t';dpp=5; 
   tim0=1000;   
   setting=set_amp; 
   stt_trg=0;
   tim3 = 10000; 
  } 
 
   
  if(btn[0]==prs && btn[1]==prs && !skb_trg)
  { 
   skb_trg=1;   
   stt_trg=0; 
   tim3 = 5000;
   
   if     (setting == set_amp)    setting = set_offset; 
   else if(setting == set_offset) setting = set_amp;//setting = set_irres;
   else                           setting = set_amp;
  }
  
  if(!tim0)
  { 
   if(!tim3){dev_st=start; goto endofsetup;} 
   
   //   
   if(setting == set_amp)
   { 
    if(!stt_trg)
    {
     stt_trg=1;
     dbuf[2]='A';dbuf[1]='P';dbuf[0]='f';dpp=5; 
    }  
    
    tmp=pm.k_amplifier;          
    
    if(!skb_trg)kb_input(&pm.k_amplifier,10000,20000,100,100,0xff/2);  
    
    if(tmp!=pm.k_amplifier) 
    {
     tim3 = 5000;
     convert_data(pm.k_amplifier/100);
    }    
   }         
   //   
   else if(setting == set_offset) 
   {
    if(!stt_trg)
    {
     stt_trg=1;
     dbuf[2]='O';dbuf[1]='f';dbuf[0]='S';dpp=5;        
    }
    
    tmp=pm.offset; 
    
    if(!skb_trg)kb_input(&pm.offset,0,9000,100,100,0xff/2); 
    
    if(tmp!=pm.offset)
    {
     tim3 = 5000;
     convert_data(pm.offset/10),dbuf[0]='C',dpp=1;
    }     
   }  
   //   
   else
   {
    if(!stt_trg)
    {
     stt_trg=1;
     dbuf[2]='S';dbuf[1]='i';dbuf[0]='r';dpp=1; 
    }
    
    tmp=pm.r_irn; 
    
    if(!skb_trg)kb_input(&pm.r_irn,1,0xff,1,1,0xff/2);
    
    if(tmp!=pm.r_irn) 
    {
     tim3 = 5000;
     convert_data(pm.r_irn);
    }     
   }  
  } 

endofsetup: 
  if(dev_st!=setup)
  {
   itr=0;
   
   if(ee.k_amplifier != pm.k_amplifier)
   {
    ee.k_amplifier   = pm.k_amplifier;
    ee.maxtc         = ADCMAX*1000*100/K_PAIR*100/pm.k_amplifier;
    ee.cpadc         = ee.maxtc/0x3ff; 
   }  
   
   if(ee.offset != pm.offset)
   {
    ee.offset = pm.offset;   
   }
   
   if(ee.r_irn != pm.r_irn)
   {
    ee.r_irn = pm.r_irn;
   }
  }
 } 
 break;   
}
}